@@ -38,7 +38,7 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
-#include "magick/cache.h"
+#include "magick/pixel_cache.h"
 #include "magick/color.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
@@ -369,7 +369,11 @@ static MagickPassFail DecodeImage(Image *image,const long opacity)
   MagickFreeMemory(prefix);
   MagickFreeMemory(packet);
   if ((status == MagickFail) || (y < (long) image->rows))
-    ThrowBinaryException(CorruptImageError,CorruptImage,image->filename);
+    {
+      if (image->exception.severity < ErrorException)
+        ThrowException(&image->exception,CorruptImageError,CorruptImage,image->filename);
+      return MagickFail;
+    }
   return(MagickPass);
 }
 
@@ -407,11 +411,8 @@ static MagickPassFail DecodeImage(Image *image,const long opacity)
 #define MaxCode(number_bits)  ((1 << (number_bits))-1)
 #define MaxHashTable  5003
 #define MaxGIFBits  12
-#if defined(HasLZW)
-#  define MaxGIFTable  (1 << MaxGIFBits)
-#else
-#  define MaxGIFTable  max_code
-#endif
+#define MaxGIFTable  (1 << MaxGIFBits)
+
 #define GIFOutputCode(code) \
 { \
   /*  \
@@ -451,10 +452,8 @@ static MagickPassFail EncodeImage(const ImageInfo *image_info,Image *image,
 {
 
   int
-#if defined(HasLZW)
     displacement,
     next_pixel,
-#endif
     bits,
     byte_count,
     k,
@@ -546,7 +545,6 @@ static MagickPassFail EncodeImage(const ImageInfo *image_info,Image *image,
       k=(int) ((int) index << (MaxGIFBits-8))+waiting_code;
       if (k >= MaxHashTable)
         k-=MaxHashTable;
-#if defined(HasLZW)
       next_pixel=False;
       displacement=1;
       if ((image_info->compression != NoCompression) && (hash_code[k] > 0))
@@ -575,7 +573,6 @@ static MagickPassFail EncodeImage(const ImageInfo *image_info,Image *image,
           if (next_pixel == True)
             continue;
         }
-#endif
       GIFOutputCode(waiting_code);
       if (free_code < MaxGIFTable)
         {
@@ -888,9 +885,11 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           GIF Extension block.
         */
         count=ReadBlob(image,1,(char *) &c);
-        if (count == 0)
-          ThrowReaderException(CorruptImageError,UnableToReadExtensionBlock,
+        if (count == 0) {
+	  MagickFreeMemory(global_colormap);
+          ThrowReaderException(CorruptImageError,UnableToReadExtensionBlock,	 
             image);
+	}
         switch (c)
         {
           case 0xf9:
@@ -994,13 +993,17 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     delay=0;
     dispose=0;
     iterations=1;
-    if ((image->columns == 0) || (image->rows == 0))
+    if ((image->columns == 0) || (image->rows == 0)) {
+      MagickFreeMemory(global_colormap);    
       ThrowReaderException(CorruptImageError,NegativeOrZeroImageSize,image);
+    }
     /*
       Inititialize colormap.
     */
-    if (!AllocateImageColormap(image,image->colors))
+    if (!AllocateImageColormap(image,image->colors)) {
+      MagickFreeMemory(global_colormap);    
       ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+    }
     if (!BitSet(flag,0x80))
       {
         /*
@@ -1048,8 +1051,11 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Decode image.
     */
     status=DecodeImage(image,opacity);
-    if (!image_info->ping && (status == False))
+    if (!image_info->ping && (status == False)) {
+      MagickFreeMemory(global_colormap);
+      GetImageException(image,exception);
       ThrowReaderException(CorruptImageError,CorruptImage,image);
+    }
     if (image_info->subrange != 0)
       if (image->scene >= (image_info->subimage+image_info->subrange-1))
         break;
@@ -1096,11 +1102,10 @@ ModuleExport void RegisterGIFImage(void)
   entry->encoder=(EncoderHandler) WriteGIFImage;
   entry->magick=(MagickHandler) IsGIF;
   entry->description=AcquireString("CompuServe graphics interchange format");
-#if !defined(HasLZW)
-  entry->version=AcquireString("LZW disabled");
-#endif
+  entry->version=AcquireString("version 89a");
   entry->module=AcquireString("GIF");
   (void) RegisterMagickInfo(entry);
+
   entry=SetMagickInfo("GIF87");
   entry->decoder=(DecoderHandler) ReadGIFImage;
   entry->encoder=(EncoderHandler) WriteGIFImage;
@@ -1295,7 +1300,7 @@ static MagickPassFail WriteGIFImage(const ImageInfo *image_info,Image *image)
               Set transparent pixel.
             */
             opacity=(long) image->colors++;
-            MagickReallocMemory(image->colormap,
+            MagickReallocMemory(PixelPacket *,image->colormap,
               image->colors*sizeof(PixelPacket));
             if (image->colormap == (PixelPacket *) NULL)
               {
