@@ -38,7 +38,7 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
-#include "magick/cache.h"
+#include "magick/pixel_cache.h"
 #include "magick/color.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
@@ -208,6 +208,8 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (header.header_size < sz_XWDheader)
     ThrowReaderException(CorruptImageError,CorruptImage,image);
   length=header.header_size-sz_XWDheader;
+  if (length > ((~0UL)/sizeof(*comment)))
+    ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
   comment=MagickAllocateMemory(char *,length+1);
   if (comment == (char *) NULL)
     ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
@@ -239,6 +241,13 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ximage->red_mask=header.red_mask;
   ximage->green_mask=header.green_mask;
   ximage->blue_mask=header.blue_mask;
+  /* Why those are signed ints is beyond me. */
+  if (ximage->depth < 0 || ximage->width < 0 || ximage->height < 0 ||
+      ximage->bitmap_pad < 0 || ximage->bytes_per_line < 0)
+    ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+  /* Guard against buffer overflow in libX11. */
+  if (ximage->bits_per_pixel > 32 || ximage->bitmap_unit > 32)
+    ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
   status=XInitImage(ximage);
   if (status == False)
     ThrowReaderException(CorruptImageError,UnrecognizedXWDHeader,image);
@@ -251,7 +260,10 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       XWDColor
         color;
 
-      colors=MagickAllocateMemory(XColor *,header.ncolors*sizeof(XColor));
+      length=(size_t) header.ncolors;
+      if (length > ((~0UL)/sizeof(*colors)))
+        ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+      colors=MagickAllocateArray(XColor *,length,sizeof(XColor));
       if (colors == (XColor *) NULL)
         ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
       for (i=0; i < (long) header.ncolors; i++)
@@ -281,10 +293,17 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Allocate the pixel buffer.
   */
-  if (ximage->format == ZPixmap)
-    length=ximage->bytes_per_line*ximage->height;
-  else
-    length=ximage->bytes_per_line*ximage->height*ximage->depth;
+#define XWD_OVERFLOW(c,a,b) ((b) != 0 && ((c)/((size_t) b) != ((size_t) a)))
+  length=ximage->bytes_per_line*ximage->height;
+  if (XWD_OVERFLOW(length,ximage->bytes_per_line,ximage->height))
+    ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+  if (ximage->format != ZPixmap)
+    {
+      size_t tmp=length;
+      length*=ximage->depth;
+      if (XWD_OVERFLOW(length,tmp,ximage->depth))
+        ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+    }
   ximage->data=MagickAllocateMemory(char *,length);
   if (ximage->data == (char *) NULL)
     ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
@@ -549,9 +568,6 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register long
     x;
 
@@ -589,7 +605,7 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
     ThrowWriterException(FileOpenError,UnableToOpenFile,image);
-  TransformColorspace(image,RGBColorspace);
+  (void) TransformColorspace(image,RGBColorspace);
   /*
     Initialize XWD file header.
   */
@@ -646,7 +662,7 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
       /*
         Dump colormap to file.
       */
-      colors=MagickAllocateMemory(XColor *,image->colors*sizeof(XColor));
+      colors=MagickAllocateArray(XColor *,image->colors,sizeof(XColor));
       if (colors == (XColor *) NULL)
         ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
       for (i=0; i < (long) image->colors; i++)
@@ -682,6 +698,7 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
   pixels=MagickAllocateMemory(unsigned char *,pixels_size);
   if (pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
+  (void) memset(pixels,0,pixels_size);
   /*
     Convert MIFF to XWD raster pixels.
   */
@@ -690,11 +707,14 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
     q=pixels;
 
     if (image->storage_class == PseudoClass)
       {
+        register IndexPacket
+          *indexes;
+
+        indexes=GetIndexes(image);
         for (x=(long) image->columns; x > 0; x--)
           *q++=(unsigned char) *indexes++;
       }
