@@ -158,25 +158,31 @@ MagickExport double ExpandAffine(const AffineMatrix *affine)
 %
 %
 */
-#define NoiseScale     ((double) MaxRGB/255)
 #define NoiseEpsilon   1.0e-5
-#define SigmaUniform   4.0*NoiseScale
-#define SigmaGaussian  4.0*NoiseScale
+#define SigmaUniform   4.0
+#define SigmaGaussian  4.0
 #define SigmaImpulse   0.10
-#define SigmaLaplacian 10.0*NoiseScale
-#define SigmaMultiplicativeGaussian  0.5*NoiseScale
+#define SigmaLaplacian 10.0
+#define SigmaMultiplicativeGaussian  0.5
 #define SigmaPoisson   0.05
-#define TauGaussian    20.0*NoiseScale
+#define TauGaussian    20.0
 
-MagickExport Quantum GenerateNoise(const Quantum pixel,
-  const NoiseType noise_type)
+MagickExport Quantum GenerateNoise(const Quantum quantum_pixel,
+                                   const NoiseType noise_type)
 {
   double
     alpha,
     beta,
+    pixel,
     sigma,
     value;
 
+  pixel=(double) quantum_pixel;
+
+#if QuantumDepth > 8
+  pixel /= MaxRGBDouble/255.0;
+#endif
+
   alpha=(double) rand()/RAND_MAX;
   if (alpha == 0.0)
     alpha=1.0;
@@ -203,7 +209,7 @@ MagickExport Quantum GenerateNoise(const Quantum pixel,
     case MultiplicativeGaussianNoise:
     {
       if (alpha <= NoiseEpsilon)
-        sigma=MaxRGB;
+        sigma=255.0;
       else
         sigma=sqrt(-2.0*log(alpha));
       beta=(double) rand()/RAND_MAX;
@@ -217,7 +223,7 @@ MagickExport Quantum GenerateNoise(const Quantum pixel,
         value=0;
        else
          if (alpha >= (1.0-(SigmaImpulse/2.0)))
-           value=MaxRGB;
+           value=255.0;
          else
            value=pixel;
       break;
@@ -227,33 +233,42 @@ MagickExport Quantum GenerateNoise(const Quantum pixel,
       if (alpha <= 0.5)
         {
           if (alpha <= NoiseEpsilon)
-            value=(double) pixel-MaxRGB;
+            value=(double) pixel-255.0;
           else
             value=(double) pixel+SigmaLaplacian*log(2.0*alpha);
           break;
         }
       beta=1.0-alpha;
       if (beta <= (0.5*NoiseEpsilon))
-        value=(double) pixel+MaxRGB;
+        value=(double) pixel+255.0;
       else
         value=(double) pixel-SigmaLaplacian*log(2.0*beta);
       break;
     }
     case PoissonNoise:
     {
+      double
+        limit;
+
       register long
         i;
 
-      for (i=0; alpha > exp(-SigmaPoisson*pixel); i++)
+      limit=exp(-SigmaPoisson*(double) pixel);
+      for (i=0; alpha > limit; i++)
       {
         beta=(double) rand()/RAND_MAX;
         alpha=alpha*beta;
       }
-      value=(i/SigmaPoisson)*NoiseScale;
+      value=(double) i/SigmaPoisson;
       break;
     }
   }
-  return (RoundSignedToQuantum(value));
+
+#if QuantumDepth > 8
+  value *= (MaxRGBFloat/255.0);
+#endif
+
+  return (RoundDoubleToQuantum(value));
 }
 
 /*
@@ -503,13 +518,12 @@ MagickExport void HWBTransform(const double hue,const double whiteness,
   v=1.0-blackness;
   if (hue == 0.0)
     {
-      *red=(Quantum) (MaxRGB*v+0.5);
-      *green=(Quantum) (MaxRGB*v+0.5);
-      *blue=(Quantum) (MaxRGB*v+0.5);
+      v *= MaxRGB;
+      *red=*green=*blue=RoundSignedToQuantum(v);
       return;
     }
-  i=(unsigned int) floor(hue);
-  f=hue-i;
+  i=(unsigned int) (6.0*hue);
+  f=6.0*hue-i;
   if (i & 0x01)
     f=1.0-f;
   n=whiteness+f*(v-whiteness);  /* linear interpolation */
@@ -524,9 +538,12 @@ MagickExport void HWBTransform(const double hue,const double whiteness,
     case 4: r=n; g=whiteness; b=v; break;
     case 5: r=v; g=whiteness; b=n; break;
   }
-  *red=(Quantum) (MaxRGB*r+0.5);
-  *green=(Quantum) (MaxRGB*g+0.5);
-  *blue=(Quantum) (MaxRGB*b+0.5);
+  r *= MaxRGB;
+  g *= MaxRGB;
+  b *= MaxRGB;
+  *red=RoundSignedToQuantum(r);
+  *green=RoundSignedToQuantum(g);
+  *blue=RoundSignedToQuantum(b);
 }
 
 /*
@@ -928,45 +945,38 @@ MagickExport void TransformHSL(const Quantum red,const Quantum green,
 %
 %
 */
-MagickExport void TransformHWB(const Quantum red,const Quantum green,
-  const Quantum blue,double *hue_result,double *whiteness_result,double *blackness_result)
+MagickExport void TransformHWB(const Quantum red,const Quantum green, const Quantum blue,
+                               double *hue,double *whiteness,double *blackness)
 {
   double
-    blackness,
     f,
-    hue,
-    whiteness;
+    v,
+    w;
 
   register long
     i;
 
-  Quantum
-    v,
-    w;
-
   /*
     Convert RGB to HWB colorspace.
   */
-  assert(hue_result != (double *) NULL);
-  assert(whiteness_result != (double *) NULL);
-  assert(blackness_result != (double *) NULL);
-  w=Min(red,Min(green,blue));
-  v=Max(red,Max(green,blue));
-  blackness=(double) (MaxRGB-v)/MaxRGB;
+  assert(hue != (double *) NULL);
+  assert(whiteness != (double *) NULL);
+  assert(blackness != (double *) NULL);
+  w=(double) Min(red,Min(green,blue));
+  v=(double) Max(red,Max(green,blue));
+  *blackness=(double) (MaxRGB-v)/MaxRGB;
   if (v == w)
     {
-      hue=0.0;
-      whiteness=1.0-(blackness);
+      *hue=0.0;
+      *whiteness=1.0-(*blackness);
     }
   else
     {
-      f=(red == w) ? (double) green-blue : ((green == w) ? (double) blue-red :
-                                            (double) red-green);
+      f=(red == w) ? (double) green-blue :
+        ((green == w) ? (double) blue-red :
+         (double) red-green);
       i=(red == w) ? 3 : ((green == w) ? 5 : 1);
-      hue=i-f/(v-w);
-      whiteness=(double) w/MaxRGB;
+      *hue=((double) i-f/(v-w))/6.0;
+      *whiteness=((double) w/MaxRGB);
     }
-  *hue_result=ConstrainToRange(0.0,1.0,hue);
-  *whiteness_result=ConstrainToRange(0.0,1.0,whiteness);
-  *blackness_result=ConstrainToRange(0.0,1.0,blackness);
 }
