@@ -37,7 +37,7 @@
 */
 #include "magick/studio.h"
 #if defined(SupportMagickModules)
-#if defined(WIN32) || defined(__CYGWIN__)
+#if defined(MSWINDOWS) || defined(__CYGWIN__)
 # include "magick/nt_feature.h"
 #endif
 #include "magick/blob.h"
@@ -161,7 +161,7 @@ static CoderInfo
   *RegisterModule(CoderInfo *,ExceptionInfo *),
   *SetCoderInfo(const char *);
 
-#if FUNCTION_UNUSED
+#if 0
 static const CoderInfo
   *GetCoderInfo(const char *,ExceptionInfo *);
 #endif
@@ -384,14 +384,14 @@ MagickExport MagickPassFail ExecuteModuleProcess(const char *tag,Image **image,
     method=(unsigned int (*)(Image **,const int,char **))
       lt_dlsym(handle,method_name);
 
-    LogMagickEvent(CoderEvent,GetMagickModule(),
-      "Invoking \"%.1024s\" filter module",tag);
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "Invoking \"%.1024s\" filter module",tag);
 
     /* Execute module method */
     if (method != (unsigned int (*)(Image **,const int,char **)) NULL)
       status=(*method)(image,argc,argv);
 
-    LogMagickEvent(CoderEvent,GetMagickModule(),
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
       "Returned from \"%.1024s\" filter module",tag);
 
   }
@@ -432,7 +432,7 @@ MagickExport MagickPassFail ExecuteModuleProcess(const char *tag,Image **image,
 %
 %
 */
-#if FUNCTION_UNUSED
+#if 0
 static const CoderInfo *GetCoderInfo(const char *tag,
   ExceptionInfo *exception)
 {
@@ -522,7 +522,7 @@ static MagickPassFail FindMagickModule(const char *filename,
   assert(filename != (const char *) NULL);
   assert(path != (char *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
-  (void) strncpy(path,filename,MaxTextExtent-1);
+  (void) strlcpy(path,filename,MaxTextExtent);
   
   if (InitializeModuleSearchPath(module_type,exception) == MagickFail)
     return (status);
@@ -561,9 +561,9 @@ static MagickPassFail FindMagickModule(const char *filename,
       while(MagickMapIterateNext(path_map_iterator,&key))
         {
           if (search_path)
-            ConcatenateString(&search_path,list_seperator);
-          ConcatenateString(&search_path,
-                            MagickMapDereferenceIterator(path_map_iterator,0));
+            (void) ConcatenateString(&search_path,list_seperator);
+          (void) ConcatenateString(&search_path,
+                                   MagickMapDereferenceIterator(path_map_iterator,0));
         }
       
       (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
@@ -663,7 +663,7 @@ static MagickPassFail GetModuleListForDirectory
     if (i >= (long) *max_entries)
       {
         *max_entries<<=1;
-        MagickReallocMemory(list,*max_entries*sizeof(char *));
+        MagickReallocMemory(char **,list,*max_entries*sizeof(char *));
         if (list == (char **) NULL)
           break;
       }
@@ -784,11 +784,11 @@ MagickExport void InitializeMagickModules(void)
               UnableToInitializeModuleLoader,lt_dlerror());
           ltdl_initialized=True;
         }
-      ReadModuleConfigureFile(ModuleFilename,0,&exception);
+      (void) ReadModuleConfigureFile(ModuleFilename,0,&exception);
     }
   LiberateSemaphoreInfo(&module_semaphore);
-  InitializeModuleSearchPath(MagickCoderModule,&exception);
-  InitializeModuleSearchPath(MagickFilterModule,&exception);
+  (void) InitializeModuleSearchPath(MagickCoderModule,&exception);
+  (void) InitializeModuleSearchPath(MagickFilterModule,&exception);
 
   DestroyExceptionInfo(&exception);
 }
@@ -826,7 +826,7 @@ static void AddModulePath(MagickMap path_map, unsigned int *path_index,
     key[MaxTextExtent];
 
   FormatString(key,"%u",*path_index);
-  MagickMapAddEntry(path_map,key,(void *)path,0,exception);
+  (void) MagickMapAddEntry(path_map,key,(void *)path,0,exception);
   (*path_index)++;
 }
 
@@ -939,10 +939,10 @@ MagickPassFail InitializeModuleSearchPath(MagickModuleType module_type,
             length=end-start;
           if (length > MaxTextExtent-1)
             length = MaxTextExtent-1;
-          strncpy(buffer,start,length);
+          (void) strncpy(buffer,start,length);
           buffer[length]='\0';
           if (buffer[length-1] != DirectorySeparator[0])
-            strcat(buffer,DirectorySeparator);
+            (void) strcat(buffer,DirectorySeparator);
           AddModulePath(path_map,&path_index,buffer,exception);
           start += length+1;
         }
@@ -973,7 +973,7 @@ MagickPassFail InitializeModuleSearchPath(MagickModuleType module_type,
     return (status);
   }
 # else
-#  if defined(WIN32)
+#  if defined(MSWINDOWS)
   {
     /*
       Locate path via registry key.
@@ -1005,10 +1005,10 @@ MagickPassFail InitializeModuleSearchPath(MagickModuleType module_type,
     LiberateSemaphoreInfo(&module_semaphore);
     return (status);
   }
-#  endif /* defined(WIN32) */
+#  endif /* defined(MSWINDOWS) */
 # endif /* !defined(MagickCoderModulesPath) */
-# if !defined(MagickCoderModulesPath) && !defined(WIN32)
-#  error MagickCoderModulesPath or WIN32 must be defined when UseInstalledMagick is defined
+# if !defined(MagickCoderModulesPath) && !defined(MSWINDOWS)
+#  error MagickCoderModulesPath or MSWINDOWS must be defined when UseInstalledMagick is defined
 # endif
 #else /* end defined(UseInstalledMagick) */
   if (getenv("MAGICK_HOME") != (char *) NULL)
@@ -1059,7 +1059,7 @@ MagickPassFail InitializeModuleSearchPath(MagickModuleType module_type,
           break;
         }
 
-      (void) strncpy(prefix,SetClientPath((char *) NULL),MaxTextExtent-1);
+      (void) strlcpy(prefix,SetClientPath((char *) NULL),MaxTextExtent);
       ChopPathComponents(prefix,1);
       FormatString(path,"%.512s/lib/%s/modules-Q%d/%s/",prefix,
         MagickLibSubdir,QuantumDepth,module_subdir);
@@ -1271,12 +1271,12 @@ MagickExport unsigned int OpenModule(const char *module,
       Assign module name from alias.
     */
     assert(module != (const char *) NULL);
-    (void) strncpy(module_name,module,MaxTextExtent-1);
+    (void) strlcpy(module_name,module,MaxTextExtent);
     if (module_list != (ModuleInfo *) NULL)
       for (p=module_list; p != (ModuleInfo *) NULL; p=p->next)
         if (LocaleCompare(p->magick,module) == 0)
           {
-            (void) strncpy(module_name,p->name,MaxTextExtent-1);
+            (void) strlcpy(module_name,p->name,MaxTextExtent);
             break;
           }
 
@@ -1312,8 +1312,8 @@ MagickExport unsigned int OpenModule(const char *module,
     if ((module_list != (ModuleInfo *) NULL) &&
         (module_list->path != (char *) NULL))
       GetPathComponent(module_list->path,HeadPath,path);
-    (void) strcat(path,DirectorySeparator);
-    (void) strncat(path,module_file,MaxTextExtent-strlen(path)-1);
+    (void) strlcat(path,DirectorySeparator,MaxTextExtent);
+    (void) strlcat(path,module_file,MaxTextExtent);
 #endif
     /*
       Load module
@@ -1509,7 +1509,7 @@ static unsigned int ReadModuleConfigureFile(const char *basename,
     GetToken(q,&q,token);
     if (*token == '\0')
       break;
-    (void) strncpy(keyword,token,MaxTextExtent-1);
+    (void) strlcpy(keyword,token,MaxTextExtent);
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
@@ -1526,7 +1526,7 @@ static unsigned int ReadModuleConfigureFile(const char *basename,
         */
         while ((*token != '>') && (*q != '\0'))
         {
-          strncpy(keyword,token,MaxTextExtent-1);
+          (void) strlcpy(keyword,token,MaxTextExtent);
           GetToken(q,&q,token);
           if (*token != '=')
             continue;
@@ -1542,9 +1542,8 @@ static unsigned int ReadModuleConfigureFile(const char *basename,
 
                   GetPathComponent(path,HeadPath,filename);
                   if (*filename != '\0')
-                    (void) strcat(filename,DirectorySeparator);
-                  (void) strncat(filename,token,MaxTextExtent-
-                    strlen(filename)-1);
+                    (void) strlcat(filename,DirectorySeparator,MaxTextExtent);
+                  (void) strlcat(filename,token,MaxTextExtent);
                   (void) ReadModuleConfigureFile(filename,depth+1,exception);
                 }
               if (module_list != (ModuleInfo *) NULL)
@@ -1800,9 +1799,9 @@ static void TagToCoderModuleName(const char *tag,char *module_name)
   (void) FormatString(module_name,"%.1024s.la",tag);
   (void) LocaleLower(module_name);
 #else
-#if defined(WIN32)
+#if defined(MSWINDOWS)
   if (LocaleNCompare("IM_MOD_",tag,7) == 0)
-    (void) strncpy(module_name,tag,MaxTextExtent-1);
+    (void) strlcpy(module_name,tag,MaxTextExtent);
   else
     {
 #  if defined(_DEBUG)
@@ -1811,7 +1810,7 @@ static void TagToCoderModuleName(const char *tag,char *module_name)
       FormatString(module_name,"IM_MOD_RL_%.1024s_.dll",tag);
 #  endif /* defined(_DEBUG) */
     }
-#endif  /* defined(WIN32) */
+#endif  /* defined(MSWINDOWS) */
 #endif /* defined(HasLTDL) */
 }
 
@@ -1889,13 +1888,14 @@ static void TagToFunctionName(const char *tag,const char *format,char *function)
   assert(tag != (const char *) NULL);
   assert(format != (const char *) NULL);
   assert(function != (char *) NULL);
-  strncpy(function_name,tag,MaxTextExtent-1);
+  (void) strlcpy(function_name,tag,MaxTextExtent);
   LocaleUpper(function_name);
 
 #if defined(PREFIX_MAGICK_SYMBOLS)
-  snprintf(extended_format,MaxTextExtent-1,"Gm%.1024s",format);
+  (void) strlcpy(extended_format,"Gm",MaxTextExtent);
+  (void) strlcat(extended_format,format,MaxTextExtent);
 #else
-  strncpy(extended_format,format,MaxTextExtent-1);
+  (void) strlcpy(extended_format,format,MaxTextExtent);
 #endif
   FormatString(function,extended_format,function_name);
 }
@@ -1941,19 +1941,25 @@ static unsigned int UnloadModule(const CoderInfo *coder_info,
   assert(coder_info->unregister_function != (void (*)(void)) NULL);
   assert(exception != (ExceptionInfo *) NULL);
 
+  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+                        "Unloading \"%s\" module ...", coder_info->tag);
+
   /*
     Invoke module unregister (UnregisterFORMATImage) function
   */
   coder_info->unregister_function();
 
   /*
-    Close module
+    Close module.  Don't close JP2 module since it uses atexit().
   */
-  if(lt_dlclose((ModuleHandle) coder_info->handle))
+  if ( strcmp("JP2",coder_info->tag) != 0 )
     {
-      FormatString(message,"\"%.1024s: %.1024s\"",name,lt_dlerror());
-      ThrowException(exception,ModuleError,FailedToCloseModule,message);
-      status=False;
+      if (lt_dlclose((ModuleHandle) coder_info->handle))
+        {
+          FormatString(message,"\"%.1024s: %.1024s\"",name,lt_dlerror());
+          ThrowException(exception,ModuleError,FailedToCloseModule,message);
+          status=False;
+        }
     }
   return (status);
 }
