@@ -4,17 +4,18 @@
 # This file is sourced by a Bourne shell (/bin/sh) script so it must
 # observe Bourne shell syntax.
 #
+
 # Package base name
 PACKAGE_NAME='GraphicsMagick'
 #
 # Package base version.  This is is the numeric version suffix applied to
-# PACKAGE_NAME (e.g. "1.1").
-PACKAGE_VERSION='1.1.10'
+# PACKAGE_NAME (e.g. "1.2").
+PACKAGE_VERSION='1.2.5'
 
 #
 # Formal Package release date
 # Set to string "unreleased" if package is not a formal release.
-PACKAGE_RELEASE_DATE="2007-09-19"
+PACKAGE_RELEASE_DATE="2008-07-25"
 #PACKAGE_RELEASE_DATE="unreleased"
 
 #
@@ -26,7 +27,7 @@ PACKAGE_CHANGE_DATE=`awk '/^[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]/ { print
 # Package version addendum.  This is a suffix (if any) appended to the
 # package base version.  Formal releases do not have a suffix applied.
 #
-if test "$PACKAGE_RELEASE_DATE" = ""
+if test "$PACKAGE_RELEASE_DATE" = "unreleased"
 then
   PACKAGE_VERSION_ADDENDUM=".0${PACKAGE_CHANGE_DATE}"
 else
@@ -36,7 +37,7 @@ fi
 #
 # CVS branch that this release is on.
 #
-CVS_BRANCH_TAG=GraphicsMagick-1_1
+CVS_BRANCH_TAG=HEAD
 
 #
 # Libtool library revision control info
@@ -62,20 +63,20 @@ CVS_BRANCH_TAG=GraphicsMagick-1_1
 #
 # Magick library versioning
 #
-MAGICK_LIBRARY_CURRENT=1
-MAGICK_LIBRARY_REVISION=10
+MAGICK_LIBRARY_CURRENT=2
+MAGICK_LIBRARY_REVISION=5
 MAGICK_LIBRARY_AGE=0
 
 #
 # Magick++ library versioning
 #
-MAGICK_PLUS_PLUS_LIBRARY_CURRENT=1
-MAGICK_PLUS_PLUS_LIBRARY_REVISION=3
+MAGICK_PLUS_PLUS_LIBRARY_CURRENT=2
+MAGICK_PLUS_PLUS_LIBRARY_REVISION=0
 MAGICK_PLUS_PLUS_LIBRARY_AGE=0
 
 #
 # Magick Wand library versioning
 #
-MAGICK_WAND_LIBRARY_CURRENT=0
-MAGICK_WAND_LIBRARY_REVISION=4
+MAGICK_WAND_LIBRARY_CURRENT=1
+MAGICK_WAND_LIBRARY_REVISION=1
 MAGICK_WAND_LIBRARY_AGE=0
